// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 - 2007 Simon Goodall

package WFUT;

import java.io.*;
import java.util.*;
import java.awt.*;

class Client {
  
  public static final String VERSION = "1.1";
   
  public static final String CMD_list = "list";
  public static final String CMD_update = "update";
  public static final String CMD_serverUpdate = "serverupdate";
  public static final String CMD_destination = "dest";
  public static final String CMD_reference = "ref";
  public static final String KEY_serverRoot = "server";
  public static final String KEY_prefix = "prefix";
  //public static final String channelDir = "channels/";

  public static final String KEY_proxyHost = "proxyHost";
  public static final String KEY_proxyPort = "proxyPort";
  

  public static void main(String arg[]) {
    System.out.println("WFUT Version: " + VERSION);
    if (arg.length == 0) {
      try {
        GUI gui = new GUI();
        gui.pack();

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension windowSize = gui.getSize();
        gui.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
                        Math.max(0,(screenSize.height-windowSize.height)/2));
        
        gui.setVisible(true);

      } catch (Exception ex) {
        ex.printStackTrace();
//        System.err.println(ex);
      } catch (Error err) {
        err.printStackTrace();
//        System.err.println(err);
      }
      return;
    }
    int counter = 0;
    String ref = null;
    while (counter < arg.length) {
      final String cmd = arg[counter++].toLowerCase();
      try {
        if (cmd.equals(CMD_destination)) {
          Options.getInstance().setPrefix(arg[counter++] + File.separator);
        }
        else if (cmd.equals(CMD_reference)) {
            ref = arg[counter++] + File.separator;
          }
        else if (cmd.equals(CMD_list)) {
          System.out.println("Fetching channel list");
          System.out.println("=====================");
          System.out.println();
          java.util.List l = Client.getInstance().listChannels();
          Iterator itr = l.iterator();
          while (itr.hasNext()) {
            final Channel channel = (Channel)itr.next();
            channel.print();
            System.out.println();
          }
        }
	else if(cmd.equals(CMD_serverUpdate)) {
          if (counter >= arg.length) {
            System.err.println("No channels specified");
            return;
          }
          while (counter < arg.length) {
            serverUpdate(arg[counter++]);
          }
        } else if(cmd.equals(CMD_update)) {
          if (counter >= arg.length) {
            System.err.println("No channels specified");
            return;
          }
          System.out.println("Updating channels");
          System.out.println("=================");
          System.out.println();
          final String server = Options.getInstance().getServerRoot();
          UpdateThread ut = new UpdateThread();
          ut.addCallbackHandler(new BasicWFUTCallbackHandler());
          while (counter < arg.length) {            
            ut.updateChannel(server, arg[counter++], ref);            
          }
        } else {
          System.err.println("Unknown command");
        } 
      } catch(Exception e) {
        System.err.println(e);
      }
    }
  }

  private Client() {
    // Configure proxy if required.
    if (Options.getInstance().getUseProxyBoolean()) {
      final String pHost = Options.getInstance().getProxyHost();
      final String pPort = Options.getInstance().getProxyPort();
      System.out.println("Using Proxy: " + pHost + ":" + pPort);
      System.setProperty("http.proxyHost", pHost);
      System.setProperty("http.proxyPort", pPort);
    }
  }

  public java.util.List listChannels() throws Exception{
    final String server = Options.getInstance().getServerRoot();
    return FileIO.getChannels(server + "channels.xml");
  }
  
  public static Client getInstance() { return instance; }
  private static Client instance = new Client();

  public static void listDir(java.util.List list, File dir) {
    File files[] = dir.listFiles();
    for (int i = 0; i < files.length; ++i) {
      if (files[i].isFile()) list.add(files[i].toString());
      else if (files[i].isDirectory()) listDir(list, files[i]);
    }
  }

  public static void serverUpdate(final String channel) {
    System.out.println("Updating serverlist: " + channel);
    java.util.List writeFiles = new ArrayList();
    try {
      Map channelList = null;
      try {
        channelList = FileIO.getLocalList(channel + "/wfut.xml").getFiles();
      } catch(Exception e) {
        channelList = new HashMap();
      }
      java.util.List files = new ArrayList();
      listDir(files, new File(channel));
      Iterator itr = files.iterator();
      while (itr.hasNext()) {
        final String filename = (String)itr.next();
        // Need to strip out channel path prefix
        final String ch_filename = filename.substring(filename.indexOf(channel) + channel.length() + 1);
        
        // Ignore wfut.xml files.
        if (ch_filename.equals("wfut.xml")) continue;
        long crc = CRC32Calc.calcCRC32(filename);
        FileObject fo = new FileObject();
        fo.setFilename(ch_filename);
        fo.setFileSize(((new File(filename)).length()));
        if (channelList.containsKey(ch_filename)) {
          FileObject f = (FileObject)channelList.get(ch_filename);
          if (f.getCRC32() != crc) {
            fo.setVersion(f.getVersion() + 1);
            fo.setExecute(f.getExecute());
            System.out.println("Updating file: " + ch_filename);
          } else {
            fo.setVersion(f.getVersion());
            fo.setExecute(f.getExecute());
          }
        } else {
          System.out.println("Adding file: " + ch_filename);
          fo.setVersion(1);

          
          // TODO: This is a big hack! This should be replace by a native call 
          // or we could make an options/metadata file that a user can edit.
          try {
            // Run prog to determine file perms
            String args[] = new String[3];
            args[0] = "ls";
            args[1] = "-l";
            args[2] = filename;

            Process p = Runtime.getRuntime().exec(args);
            BufferedReader br = new BufferedReader(new InputStreamReader(p.getInputStream()));
            //Wait for process to complete
            p.waitFor();
            // Read result
            final String line = br.readLine();
            // Check to see if user execute permission is set
            if (line.charAt(3) == 'x') {
              fo.setExecute(true);
            }
            br.close();
          } catch (java.io.IOException ioe) {
            // Generally ignore
            System.out.println(ioe); 
          }
        }
        fo.setCRC32(crc);
        
        writeFiles.add(fo);
      }

      FileWriter output = new FileWriter(channel + "/wfut.xml");
      FileObjectWriter.writeFileHeader(output, channel);

      itr = writeFiles.iterator();
      while (itr.hasNext()) {
        FileObject fo = (FileObject)itr.next();
        FileObjectWriter.writeFileObject(output, fo);
      }
      FileObjectWriter.writeFileFooter(output);
      output.close();

    } catch(Exception e) {
      System.err.println(e);
    }
  }
}
